#include "indent-toolbox.h"

#include <gtk/gtk.h>

#include <libgwymodule/gwymodule-file.h>
#include <app/file.h>

#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <locale.h>

#include "indent-toolbox-common.h"

#include "datatypes.h"
#include "controls.h"

#include "fddata.h"
#include "file-utils.h"
#include "gui-utils.h"

#ifndef G_OS_WIN32
#include "resources.h"
#endif

#include "tool_contact.h"

#include "tool_op.h"
#include "tool_op_unc.h"
#include "tool_op_mc.h"
#ifndef NOFORTRAN
#include "tool_op_odr.h"
#include "tool_op_odr_unc.h"
#include "tool_op_odr_mc.h"
#include "tool_twoslopes.h"
#endif
#include "tool_tangent.h"
#include "tool_tangent_unc.h"
#include "tool_tangent_mc.h"
#include "tool_hertz.h"
#include "tool_hertz_unc.h"
#include "tool_hertz_mc.h"
#include "tool_Ph2.h"
#include "tool_apopins.h"
#include "tool_epwork.h"
#include "tool_epwork_unc.h"
/* #include "tool_ppopins.h" */
/* #include "tool_creep.h" */
/* #include "tool_relax.h" */

#include "dialog_about.h"
#include "settings.h"
#include "xmlsettings.h"

#ifdef G_OS_WIN32
#include "getline.h"
#include "windows.h"
#endif

#define appname "Niget"

/* place somewhere else if possible */
/* keep items consitent with the enum DataFileFormat */
static const char* data_file_format_names[] = {"Niget: Preprocessed and split data",
	"UNHT: Time / Depth (nm) / Load (mN)",
	"UNHT: Time / Depth (nm) / Load (uN)",
	"Hysitron: Depth (nm) / Load (uN) / ...",
	"Other: Depth (nm) / Load (mN) / ...",
	"Other: Load (mN) / Depth (nm) / ...",
        "Other: Load (uN) / Depth (nm) / ..."};

/* zmenit na "niget.conf" nebo tak neco */
static const gchar *settings_filename = "niget_settings.cfg";

/* COPIED FROM TOOL_CONTACT.C, REMOVE AFTER CONSOLIDATION! */
static const gchar* point_sel_status[4] = {"Not set", "Auto set", "Man set", "From file"};

static void init_args(Args *args);
static void load_data (GtkFileChooserButton *widget, Data *data);
static gboolean open_file_dialog(Args *args, GtkWidget *parent);
static void show_help(void);
static void notebook_set_sensitive_all_pages(MainControls *maincontrols, gboolean val);
static void restore_zoom_all_tool_graphs(Data *data);


static void
init_args(Args *args)
{
	init_FDdata(&(args->fddata));
	init_Area(&(args->area));
	init_Instdata(&(args->instdata));
	init_OPdata(&(args->opdata));
	init_OPUncdata(&(args->opunc));
#ifndef NOFORTRAN
	init_OPODRdata(&(args->opodrdata));
	init_OPODRUncdata(&(args->opodrunc));
	init_Slopesdata(&(args->slopesdata));
	init_SlopesUncdata(&(args->slopesunc));
#endif
	init_Tangentdata(&(args->tgdata));
	init_TangentUncdata(&(args->tgunc));
	init_Hertzdata(&(args->hertzdata));
	init_HertzUncdata(&(args->hertzunc));
	init_Ph2data(&(args->ph2data));
	init_Apopindata(&(args->apopdata));
	init_Workdata(&(args->workdata));
	init_WorkUncdata(&(args->workunc));
}

static gchar*
build_tool_filename(gchar* basefile, gchar *basedir, const gchar* prefix)
{
	gchar toolprefix[100];
	gchar *toolfilename;
	gchar *fnm;

	g_snprintf(toolprefix, sizeof(toolprefix), prefix);
	toolfilename = g_malloc(1+strlen(basefile)+ strlen(toolprefix));;
	strcpy(toolfilename, basefile);
	strcat (toolfilename, toolprefix);
	fnm = g_build_filename(basedir, toolfilename, NULL);

	g_free(toolfilename);
	return fnm;
}

static gboolean
overwrite_tool_data(gchar *fnm)
{
	gboolean write  = TRUE;
	GtkWidget *dialog2;

	if (g_file_test(fnm, G_FILE_TEST_EXISTS)){
		dialog2 = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
				GTK_MESSAGE_ERROR, GTK_BUTTONS_YES_NO,
				_("File `%s' already exists. Replace?"),
				fnm);
		write = (gtk_dialog_run(GTK_DIALOG(dialog2)) == GTK_RESPONSE_YES);
		gtk_widget_destroy(dialog2);
	}
	return write;
}

static void
save_data(Data *data)
{
	GtkWidget *dialog;
	gchar *basefilename;
	gchar *basedir, *basefile;
	gchar *fnm;
	gboolean write;
	Args *args;
	Controls *ctrls;

	args = data->args;
	ctrls = data->controls;

	if (args->fddata.hall == NULL) {
	    show_error("No data loaded.");
	    return;
	}

	dialog = gtk_file_chooser_dialog_new("Save File",
			NULL,
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_SAVE, GTK_RESPONSE_OK,
			NULL);

	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), gwy_app_get_current_directory());

	if ( gtk_dialog_run (GTK_DIALOG ( dialog)) == GTK_RESPONSE_OK)
	{
		basefilename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		basefile = g_path_get_basename(basefilename);
		basedir = g_path_get_dirname(basefilename);

		// data
		fnm = build_tool_filename(basefile,basedir, "_data.txt");
		write = overwrite_tool_data(fnm);
		if (write)
		    data_write_file(fnm, args);
		g_free(fnm);

		//OP
		fnm = build_tool_filename(basefile,basedir, "_op.txt");
		write = overwrite_tool_data(fnm);
		if (write)
			op_write_file(fnm, args);
		g_free(fnm);

		//OP Uncertainties
		if (args->opunc.Ec != NULL && args->opunc.Hc !=NULL){
			fnm = build_tool_filename(basefile,basedir, "_op_unc.txt");
			write = overwrite_tool_data(fnm);
			if (write)
				op_uncertainty_write_file(fnm, args);
			g_free(fnm);

			// OP Monte Carlo
			if (ctrls->opmccontrols->has_window){
				fnm = build_tool_filename(basefile,basedir, "_op_mc.txt");
				write = overwrite_tool_data(fnm);
				if (write)
					op_mc_write_file(fnm, args);
				g_free(fnm);
			}
		}

#ifndef NOFORTRAN
		//OP ODR
		fnm = build_tool_filename(basefile,basedir, "_op_odr.txt");
		write = overwrite_tool_data(fnm);
		if (write)
			op_odr_write_file(fnm, args);
		g_free(fnm);
		
		//OP ODR Uncertainties
		if (args->opodrunc.Ec != NULL && args->opodrunc.Hc !=NULL){
			fnm = build_tool_filename(basefile,basedir, "_op_odr_unc.txt");
			write = overwrite_tool_data(fnm);
			if (write)
				op_odr_uncertainty_write_file(fnm, args);
			g_free(fnm);

			// OP ODR Monte Carlo
			if (ctrls->opodrmccontrols->has_window){
				fnm = build_tool_filename(basefile,basedir, "_op_odr_mc.txt");
				write = overwrite_tool_data(fnm);
				if (write)
					op_odr_mc_write_file(fnm, args);
				g_free(fnm);
			}
		}
		
		// Two Slopes
		fnm = build_tool_filename(basefile,basedir, "_slopes.txt");
		write = overwrite_tool_data(fnm);
		if (write)
			slopes_write_file(fnm, args);
		g_free(fnm);
#endif

		//Tangent 
		fnm = build_tool_filename(basefile,basedir, "_tangent.txt");
		write = overwrite_tool_data(fnm);
		if (write)
			tangent_write_file(fnm, args);
		g_free(fnm);
		
		//Tangent Uncertainties
		if (args->tgunc.Ec != NULL && args->tgunc.Hc !=NULL){
			fnm = build_tool_filename(basefile,basedir, "_tangent_unc.txt");
			write = overwrite_tool_data(fnm);
			if (write)
				tangent_uncertainty_write_file(fnm, args);
			g_free(fnm);

			// Tangent Monte Carlo
			if (ctrls->tgmccontrols->has_window){
				fnm = build_tool_filename(basefile,basedir, "_tangent_mc.txt");
				write = overwrite_tool_data(fnm);
				if (write)
					tangent_mc_write_file(fnm, args);
				g_free(fnm);
			}
		}
		
		// Hertz
		fnm = build_tool_filename(basefile,basedir, "_hertz.txt");
		write = overwrite_tool_data(fnm);
		if (write)
			hertz_write_file(fnm, args);
		g_free(fnm);

		//Hertz Uncertainties
		if (args->hertzunc.Ec != NULL){
			fnm = build_tool_filename(basefile,basedir, "_hertz_unc.txt");
			write = overwrite_tool_data(fnm);
			if (write)
				hertz_uncertainty_write_file(fnm, args);
			g_free(fnm);

			// Hertz Monte Carlo
			if (ctrls->hertzmccontrols->has_window){
				fnm = build_tool_filename(basefile,basedir, "_hertz_mc.txt");
				write = overwrite_tool_data(fnm);
				if (write)
					hertz_mc_write_file(fnm, args);
				g_free(fnm);
			}
		}
		
		//Diff Hard.
		fnm = build_tool_filename(basefile,basedir, "_Ph2.txt");
		write = overwrite_tool_data(fnm);
		if (write)
			Ph2_write_file(fnm, args);
		g_free(fnm);

		// Pop-in
		fnm = build_tool_filename(basefile,basedir, "_popin.txt");
		write = overwrite_tool_data(fnm);
		if (write)
			apopin_write_file(fnm, args);
		g_free(fnm);

		// Elastic-plastic Work
		fnm = build_tool_filename(basefile,basedir, "_work.txt");
		write = overwrite_tool_data(fnm);
		if (write)
			work_write_file(fnm, args);
		g_free(fnm);
		
		//Elastic- plastic Work Uncertainties
		if (args->workunc.We != NULL && args->workunc.Wp !=NULL){
			fnm = build_tool_filename(basefile,basedir, "_work_unc.txt");
			write = overwrite_tool_data(fnm);
			if (write)
				work_uncertainty_write_file(fnm, args);
			g_free(fnm);
		}

		g_free(basefilename);
		g_free(basefile);
		g_free(basedir);
	}

	gtk_widget_destroy ( dialog);
}

static void
load_data (GtkFileChooserButton *widget, Data *data)
{
    FDdata *fddata;
    ContactControls *contactcontrols;
    MainControls *maincontrols;
    
    char str[1024];
    gboolean splitok;
    FileReadStatus filestatus;
    gint timeres;

    fddata = &(data->args->fddata);
    contactcontrols = data->controls->contactcontrols;
    maincontrols = data->controls->maincontrols;

    if (open_file_dialog(data->args, NULL)) {

	filestatus = read_file(fddata);
	switch (filestatus) {

	case READ_FILE_OK:
	    backup_hF_all_data(fddata);
		    
	    if (fddata->fileformat == DATA_FILE_FORMAT_NIGET) {
		splitok = split_hF_data_spec(fddata,
					     fddata->i_contact_load, fddata->i_load_hold,
					     fddata->i_hold_unload, fddata->i_end_unload);
	    }
	    else { /* standard data formats */
		splitok = split_hF_data(fddata);

		fddata->status_load_hold = POINT_STATUS_SET_AUTO;
		fddata->status_hold_unload = POINT_STATUS_SET_AUTO;
		fddata->status_end_unload = POINT_STATUS_SET_AUTO;
	    }

	    if (!splitok)
		show_warning("Could not split file into loading and unloading.\n Proceed with care.");
			
	    printf("Data loaded: %d datapoints from file %s;\n \
					\t the loading curve has %d points;\n \
					\t the holding curve has %d points;\n \
					\t the unloading curve has %d points.\n",
		   fddata->hall->res, fddata->filename, fddata->hload->res, fddata->hhold->res, fddata->hunload->res);

	    g_snprintf(str, 1024, "%s - %s", g_get_application_name (), fddata->filename);
	    gtk_window_set_title (GTK_WINDOW (maincontrols->window_main), str);

	    update_fd_plot(contactcontrols, fddata);

	    timeres = (gint)gwy_data_line_get_res(fddata->hall_orig);
	    fddata->hall_time = gwy_data_line_duplicate(fddata->hall_orig);
	    fddata->Fall_time = gwy_data_line_duplicate(fddata->Fall_orig);

	    gwy_data_line_multiply (fddata->hall_time, 1 / fddata->hmax);
	    gwy_data_line_multiply (fddata->Fall_time, 1 / fddata->Fmax);

	    update_fd_time_plot(contactcontrols, fddata, timeres);
	    update_point_marklines (contactcontrols, fddata);

	    gwy_selection_clear (contactcontrols->selection_fd);
	    gwy_selection_clear (contactcontrols->selection_fd_time);
	    contact_switch_to_fd (data);

	    gtk_label_set_text (GTK_LABEL(contactcontrols->status_contact_load), point_sel_status[fddata->status_contact_load]);
	    gtk_label_set_text (GTK_LABEL(contactcontrols->status_load_hold), point_sel_status[fddata->status_load_hold]);
	    gtk_label_set_text (GTK_LABEL(contactcontrols->status_hold_unload), point_sel_status[fddata->status_hold_unload]);
	    gtk_label_set_text (GTK_LABEL(contactcontrols->status_end_unload), point_sel_status[fddata->status_end_unload]);

	    update_max_labels(data);
	    remove_all_tool_fit_results_labels(data, FALSE);
	    restore_zoom_all_tool_graphs(data);

	    notebook_set_sensitive_all_pages(maincontrols, TRUE);
	    gtk_notebook_set_current_page (GTK_NOTEBOOK (maincontrols->notebook_tools), CONTACT);
	    gtk_widget_set_sensitive (contactcontrols->button_save_data, TRUE);
	    break;
	    
	case READ_FILE_COULD_NOT_OPEN:
	    show_error("Could not open file.");
	    break;

	case READ_FILE_NOT_ENOUGH_ENTRIES:
	    show_error("File does not contain enough entries with specified data format.");
	    break;

	case READ_FILE_FORMAT_MISMATCH:
	    show_error("Data do not match the chosen file format.");
	    break;

	case READ_FILE_CONTAINS_NAN:
	    show_error("File contains NaN values.");
	    break;

	case READ_FILE_CONTAINS_INF:
	    show_error("File contains infinite values.");
	    break;
	} /* switch filestatus */
    } /* open_file_dialog */
    else {
	printf("No data loaded \n");
    }
}

static gboolean
open_file_dialog(Args *args, GtkWidget *parent)
{
    GtkWidget *dialog, *extra_hbox, *cbox_fileformat;
    gchar *filename;
    enum DataFileFormat fileformat;
    FileReadStatus filestatus;
    gint i, response;
    gboolean file_ok;

    dialog = gtk_file_chooser_dialog_new("Open File",
					 GTK_WINDOW(parent),
					 GTK_FILE_CHOOSER_ACTION_OPEN,
					 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					 GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
					 NULL);

    cbox_fileformat = gtk_combo_box_text_new ();
    for (i = 0; i < ndatafileformats; i++)
	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (cbox_fileformat), data_file_format_names[i]);
    gtk_combo_box_set_active (GTK_COMBO_BOX (cbox_fileformat), 0);

    extra_hbox = gtk_hbox_new (FALSE, CTRLS_SPACING);
    gtk_box_pack_end (GTK_BOX (extra_hbox), cbox_fileformat, FALSE, FALSE, 0);
    gtk_box_pack_end (GTK_BOX (extra_hbox), gtk_label_new ("Input data format"), FALSE, FALSE, 0);

    gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (dialog), extra_hbox);
    gtk_widget_show_all(extra_hbox);

    file_ok = FALSE;
    response = gtk_dialog_run (GTK_DIALOG (dialog));
    while ((response == GTK_RESPONSE_ACCEPT) && !file_ok) {
	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
	fileformat = gtk_combo_box_get_active (GTK_COMBO_BOX (cbox_fileformat));
	printf("filename %s, fileformat %d (%s) \n", filename, fileformat, data_file_format_names[fileformat]);
	
	file_ok = file_contains_data(filename, fileformat, &filestatus);
	if (file_ok) {
	    if (args->fddata.filename != NULL)
		g_free(args->fddata.filename);
	    args->fddata.filename = g_strdup(filename);
	    args->fddata.fileformat = fileformat;
	    g_free(filename);
	}
	else {
	    switch (filestatus) {
	    case READ_FILE_COULD_NOT_OPEN:
		show_error("Could not open file.");
		break;
	    
	    case READ_FILE_FORMAT_MISMATCH:
		show_error("File header does not conform to Niget data file format.");
		break;

	    case READ_FILE_NOT_ENOUGH_ENTRIES:
		show_error("File does not contain enough entries with specified data format.");
		break;
	    
	    default:
		break;
	    }

	    g_free(filename);
	    response = gtk_dialog_run (GTK_DIALOG (dialog));
	}
    }
    
    gtk_widget_destroy (dialog);
    
    if (file_ok)
	return TRUE;
    else
	return FALSE;
}

void
page_switched(Data *data, GtkNotebookPage *page, gint pageno)
{
	switch(pageno){
		case(CONTACT):
			printf("activate contact tool \n");
			break;
		case(OP):
			printf("activate op tool \n");
			op_active(data);
			break;
			
                #ifndef NOFORTRAN
		case(OPODR):
			printf("activate opodr tool \n");
			op_odr_active(data);
			break;
		#endif
			
		case(TANGENT):
			printf("activate tangent tool \n");
			tangent_active(data);
			break;
		case(HERTZ):
			printf("activate hertz tool \n");
			hertz_active(data);
			break;

                #ifndef NOFORTRAN
		case(SLOPES):
			printf("activate slopes tool \n");
			slopes_active(data);
			break;
		#endif

		case(DIFF):
			printf("activate Ph2 tool \n");
			Ph2_active(data);
			break;
		case(APOP):
			printf("activate popin-detectionk tool \n");
			apopin_active(data);
			break;
		case(WORK):
			printf("activate work tool \n");
			work_active(data);
			break;
		/* case(CREEP): */
		/* 	printf("activate creep tool \n"); */
		/* 	break; */
		/* case(RELAX): */
		/* 	printf("activate relax tool \n"); */
		/* 	break; */
		/* case(PPOP): */
		/* 	printf("activate Ppopin tool \n"); */
		/* 	ppopin_active(data); */
		/* 	break; */
	}
}

static void
quit(Data *data)
{
	save_settings(g_build_filename(g_get_user_config_dir(), settings_filename,NULL), data->args);
	gtk_main_quit();
}

static void
show_help(void)
{
#ifndef G_OS_WIN32
    GError *error = NULL;
    
    g_app_info_launch_default_for_uri("http://nanometrologie.cz/niget/doc/html4/", NULL, &error);

    if (error)
	    printf("Could not open help file, error code: %d, message %s\n", error->code, error->message);
#else
    /* Should be fixed in glib */
    GError *error = NULL;
    if (!((int)ShellExecute(NULL, "open", "http://nanometrologie.cz/niget/doc/html4/", NULL, NULL, SW_SHOWNORMAL) > 32))
	show_error("Failed to open online help.");
#endif
}

static void
notebook_set_sensitive_all_pages(MainControls *maincontrols, gboolean val)
{
    gtk_widget_set_sensitive(GTK_WIDGET(maincontrols->tab_contact), val);
    gtk_widget_set_sensitive(GTK_WIDGET(maincontrols->tab_op), val);
    #ifndef NOFORTRAN
    gtk_widget_set_sensitive(GTK_WIDGET(maincontrols->tab_op_odr), val);
    gtk_widget_set_sensitive(GTK_WIDGET(maincontrols->tab_slopes), val);
    #endif
    gtk_widget_set_sensitive(GTK_WIDGET(maincontrols->tab_tangent), val);
    gtk_widget_set_sensitive(GTK_WIDGET(maincontrols->tab_hertz), val);
    gtk_widget_set_sensitive(GTK_WIDGET(maincontrols->tab_ph2), val);
    gtk_widget_set_sensitive(GTK_WIDGET(maincontrols->tab_apopins), val);
    gtk_widget_set_sensitive(GTK_WIDGET(maincontrols->tab_epwork), val); 
 /*   gtk_widget_set_sensitive(GTK_WIDGET(maincontrols->tab_ppopins), val); */
}

static void
restore_zoom_all_tool_graphs(Data *data)
{
    op_zoom_restore(data);
    op_odr_zoom_restore(data);
    tg_zoom_restore(data);
    hz_zoom_restore(data);
    ph2_zoom_restore_Fh(data);
    ph2_zoom_restore_H(data);
    apop_zoom_restore_h(data);
    apop_zoom_restore_der(data);
    work_zoom_restore(data);
}

int
main (int argc, char **argv)
{
	Args args;
	Controls controls;
	Data data;
	MainControls maincontrols;
	ContactControls contactcontrols;
	AreaControls areacontrols;
	OPControls opcontrols;
	OPUncControls opunccontrols;
	OPMCControls opmccontrols;
#ifndef NOFORTRAN
	OPODRControls opodrcontrols;
	OPODRUncControls opodrunccontrols;
	OPODRMCControls opodrmccontrols;
	SlopesControls slopescontrols;
	SlopesUncControls slopesunccontrols;
	SlopesMCControls slopesmccontrols;
#endif
	TangentControls tgcontrols;
	TangentUncControls tgunccontrols;
	TangentMCControls tgmccontrols;
	HertzControls hzcontrols;
	HertzUncControls hzunccontrols;
	HertzMCControls hzmccontrols;
	Ph2Controls ph2controls;
	ApopinControls apopcontrols;
	WorkControls workcontrols;
	WorkUncControls workunccontrols;
	/* PpopinControls ppopcontrols; */
	/* CreepControls creepcontrols; */
	/* RelaxationControls relaxcontrols; */

	GdkScreen *screen;

	/* Initialize the widget set */
	gtk_init (&argc, &argv);
	g_set_application_name (appname);
	gwy_stock_register_stock_items (); // Gwyddion stock icons

#ifndef G_OS_WIN32
	g_resources_register(resources_get_resource());
#endif

	/* use decimal dot for all numeric data loading/writing with all system locales */
	setlocale(LC_NUMERIC, "C");

	controls.maincontrols = &maincontrols;
	controls.contactcontrols = &contactcontrols;
	controls.areacontrols = &areacontrols;
	controls.opcontrols = &opcontrols;
	controls.opunccontrols = &opunccontrols;
	controls.opmccontrols = &opmccontrols;
#ifndef NOFORTRAN
	controls.opodrcontrols = &opodrcontrols;
	controls.opodrunccontrols = &opodrunccontrols;
	controls.opodrmccontrols = &opodrmccontrols;
	controls.slopescontrols = &slopescontrols;
	controls.slopesunccontrols = &slopesunccontrols;
	controls.slopesmccontrols = &slopesmccontrols;
#endif
	controls.tgcontrols = &tgcontrols;
	controls.tgunccontrols = &tgunccontrols;
	controls.tgmccontrols = &tgmccontrols;
	controls.hertzcontrols = &hzcontrols;
	controls.hertzunccontrols = &hzunccontrols;
	controls.hertzmccontrols = &hzmccontrols;
	controls.ph2controls = &ph2controls;
	controls.apopcontrols = &apopcontrols;
	controls.workcontrols = &workcontrols;
	controls.workunccontrols = &workunccontrols;
	/* controls.ppopcontrols = &ppopcontrols; */

	init_args(&args);

	/* POKUS */
	/*save_xml_settings("settings.xml", &args);*/

	data.args = &args;
	data.controls = &controls;

	load_settings(g_build_filename(g_get_user_config_dir(), settings_filename,NULL), data.args);


	/* Main window */
	maincontrols.window_main = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(maincontrols.window_main),
			     g_strconcat (g_get_application_name(), ": NanoIndentation General Evaluation Tool", NULL));
	
	screen = gtk_window_get_screen(GTK_WINDOW(maincontrols.window_main));
	gtk_window_set_default_size(GTK_WINDOW(maincontrols.window_main),
			(gint)( 0.9 * gdk_screen_get_width(screen)), (gint) (0.9 * gdk_screen_get_height(screen)));
	gtk_window_set_position(GTK_WINDOW(maincontrols.window_main), GTK_WIN_POS_CENTER);

	g_signal_connect_swapped (G_OBJECT(maincontrols.window_main), "destroy", G_CALLBACK (quit), &data);

	/* Scrolling */
	maincontrols.scrolledwindow_main = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (maincontrols.scrolledwindow_main), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	/* Main notebook and its contents */
	maincontrols.notebook_tools = gtk_notebook_new ();
	/* g_object_set (notebook_tools, "scrollable", TRUE); */

	maincontrols.tab_contact = tool_contact_create(&data);
	maincontrols.tab_contact_label = gtk_label_new ("Data & instrument");
	gtk_notebook_append_page (GTK_NOTEBOOK (maincontrols.notebook_tools), maincontrols.tab_contact, maincontrols.tab_contact_label);

	maincontrols.tab_op = tool_op_create(&data);
	maincontrols.tab_op_label = gtk_label_new ("Oliver-Pharr analysis");
	gtk_notebook_append_page (GTK_NOTEBOOK (maincontrols.notebook_tools), maincontrols.tab_op, maincontrols.tab_op_label);

#ifndef NOFORTRAN
	maincontrols.tab_op_odr = tool_op_odr_create(&data); //
	maincontrols.tab_op_odr_label = gtk_label_new ("Oliver-Pharr analysis with ODR");
	gtk_notebook_append_page (GTK_NOTEBOOK (maincontrols.notebook_tools), maincontrols.tab_op_odr, maincontrols.tab_op_odr_label);
#endif

	maincontrols.tab_tangent = tool_tangent_create(&data); //
	maincontrols.tab_tangent_label = gtk_label_new ("Tangent analysis");
	gtk_notebook_append_page (GTK_NOTEBOOK (maincontrols.notebook_tools), maincontrols.tab_tangent, maincontrols.tab_tangent_label);

	maincontrols.tab_hertz = tool_hertz_create(&data); //
	maincontrols.tab_hertz_label = gtk_label_new ("Hertz model");
	gtk_notebook_append_page (GTK_NOTEBOOK (maincontrols.notebook_tools), maincontrols.tab_hertz, maincontrols.tab_hertz_label);

#ifndef NOFORTRAN
	maincontrols.tab_slopes = tool_slopes_create(&data); //
	maincontrols.tab_slopes_label = gtk_label_new ("Two slopes model");
	gtk_notebook_append_page (GTK_NOTEBOOK (maincontrols.notebook_tools), maincontrols.tab_slopes, maincontrols.tab_slopes_label);
#endif

	maincontrols.tab_ph2 = tool_Ph2_create(&data); //
	maincontrols.tab_ph2_label = label_new_with_markup ("P-h<sup>2</sup> analysis");
	gtk_notebook_append_page (GTK_NOTEBOOK (maincontrols.notebook_tools), maincontrols.tab_ph2, maincontrols.tab_ph2_label);

	maincontrols.tab_apopins = tool_apopins_create(&data); //
	maincontrols.tab_apopins_label = gtk_label_new ("Pop-in detection");
	gtk_notebook_append_page (GTK_NOTEBOOK (maincontrols.notebook_tools), maincontrols.tab_apopins, maincontrols.tab_apopins_label);

	maincontrols.tab_epwork = tool_epwork_create(&data); //
	maincontrols.tab_epwork_label = gtk_label_new ("Elastic/plastic work");
	gtk_notebook_append_page (GTK_NOTEBOOK (maincontrols.notebook_tools), maincontrols.tab_epwork, maincontrols.tab_epwork_label);

	/*
	maincontrols.tab_creep = tool_creep_create(&args, &controls); //
	maincontrols.tab_creep_label = gtk_label_new ("Creep");
	gtk_notebook_append_page (GTK_NOTEBOOK (maincontrols.notebook_tools), maincontrols.tab_creep, maincontrols.tab_creep_label);

	maincontrols.tab_relax = tool_relax_create(&args, &controls); //
	maincontrols.tab_relax_label = gtk_label_new ("Relaxation");
	gtk_notebook_append_page (GTK_NOTEBOOK (maincontrols.notebook_tools), maincontrols.tab_relax, maincontrols.tab_relax_label);

	maincontrols.tab_ppopins = tool_ppopins_create(&data); //
	maincontrols.tab_ppopins_label = gtk_label_new ("Pop-ins Petr");
	gtk_notebook_append_page (GTK_NOTEBOOK (maincontrols.notebook_tools), maincontrols.tab_ppopins, maincontrols.tab_ppopins_label);
	*/

	g_signal_connect_swapped(GTK_NOTEBOOK (maincontrols.notebook_tools), "switch-page", G_CALLBACK(page_switched), &data);

	/* About */
	about_dialog_create(&data);

	/* Buttons */

	maincontrols.button_open = gtk_button_new_from_stock (GTK_STOCK_OPEN);
	gtk_widget_set_tooltip_text (maincontrols.button_open, "Open file containing force-depth indentation data");
	g_signal_connect (maincontrols.button_open, "clicked", G_CALLBACK (load_data), &data);

	//maincontrols.button_save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
	maincontrols.button_save = gtk_button_new_with_mnemonic ( "Export _all data & results");
	gtk_button_set_image (GTK_BUTTON (maincontrols.button_save), gtk_image_new_from_stock (GTK_STOCK_SAVE, GTK_ICON_SIZE_BUTTON));
	gtk_widget_set_tooltip_text (maincontrols.button_save, "Export all data and results in a human-readable text format");
	g_signal_connect_swapped (maincontrols.button_save, "clicked", G_CALLBACK (save_data), &data); 

	//maincontrols.button_help = gtk_button_new_from_stock (GTK_STOCK_HELP);
	maincontrols.button_help = gtk_button_new_with_mnemonic ( "H_elp");
	gtk_button_set_image (GTK_BUTTON (maincontrols.button_help), gtk_image_new_from_stock (GTK_STOCK_HELP, GTK_ICON_SIZE_BUTTON));
	gtk_widget_set_tooltip_text (maincontrols.button_help, "Show HTML documentation in a web browser");
	g_signal_connect (maincontrols.button_help, "clicked", G_CALLBACK (show_help), NULL);

	//maincontrols.button_about = gtk_button_new_from_stock (GTK_STOCK_ABOUT);
	maincontrols.button_about = gtk_button_new_with_mnemonic ("Abou_t");
	gtk_button_set_image (GTK_BUTTON (maincontrols.button_about), gtk_image_new_from_stock (GTK_STOCK_ABOUT, GTK_ICON_SIZE_BUTTON));
	gtk_widget_set_tooltip_text (maincontrols.button_about, "Show basic information on the software");
	g_signal_connect (maincontrols.button_about, "clicked", G_CALLBACK (show_about), &data);

	maincontrols.button_quit = gtk_button_new_from_stock (GTK_STOCK_QUIT);
	g_signal_connect_swapped (maincontrols.button_quit, "clicked", G_CALLBACK (quit), &data);

	maincontrols.vbox_main = gtk_vbox_new (FALSE, CTRLS_SPACING);
	maincontrols.hbox_buttons = gtk_hbox_new (FALSE, CTRLS_SPACING);

	gtk_box_pack_start (GTK_BOX (maincontrols.hbox_buttons), maincontrols.button_open, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (maincontrols.hbox_buttons), maincontrols.button_save, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (maincontrols.hbox_buttons), maincontrols.button_help, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (maincontrols.hbox_buttons), maincontrols.button_about, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (maincontrols.hbox_buttons), maincontrols.button_quit, FALSE, FALSE, 0);

	gtk_box_pack_start (GTK_BOX (maincontrols.vbox_main), maincontrols.notebook_tools, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (maincontrols.vbox_main), maincontrols.hbox_buttons, FALSE, FALSE, 0);

	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (maincontrols.scrolledwindow_main), maincontrols.vbox_main);
	gtk_container_add (GTK_CONTAINER (maincontrols.window_main), maincontrols.scrolledwindow_main);

	notebook_set_sensitive_all_pages(&maincontrols, FALSE);
	gtk_widget_show_all (maincontrols.window_main);
	gtk_notebook_set_current_page (GTK_NOTEBOOK(maincontrols.notebook_tools), CONTACT); /* until we find what causes OP tool to be active by default */

	gtk_main ();

	return 0;
}
