#include <math.h>

#include <gtk/gtk.h>

#include <app/file.h>

#include "indent-toolbox-common.h"
#include "datatypes.h"
#include "controls.h"
#include "gui-utils.h"
#include "mc-utils.h"
#include "tool_hertz_mc.h"
#include "tool_hertz.h"


#define NPARAM 2  // Er, Eit
#define MAXBAD 5 //maximum percentage of nonsense data


void
init_HertzMCdata (HertzMCdata *hzmc, gint Nmc)
{
	gint i;

	//number of iterations
	hzmc->Nmc = Nmc;
	printf("initialize for %d iterations \n", hzmc->Nmc);

	//how much had to be skipped 
	hzmc->skipmc = 0;

	//dynamically allocate
	hzmc->hertzmc = (Hertzdata *)g_malloc(hzmc->Nmc*sizeof(Hertzdata));
	hzmc->mcavg = (gdouble *)g_malloc(NPARAM*sizeof(gdouble));
	hzmc->mcstd = (gdouble *)g_malloc(NPARAM*sizeof(gdouble));
	hzmc->mcdd = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));
	hzmc->mcxhist = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));
	hzmc->mcyhist = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));

	hzmc->mcnstat = (gint)floor(3.49*cbrt(hzmc->Nmc)+0.5);
	for (i = 0; i< NPARAM; i++){
		hzmc->mcdd[i] = (gdouble *) g_malloc(hzmc->Nmc*sizeof(gdouble));
		hzmc->mcxhist[i] = (gdouble *) g_malloc(hzmc->mcnstat*sizeof(gdouble));
		hzmc->mcyhist[i] = (gdouble *) g_malloc(hzmc->mcnstat*sizeof(gdouble));
	}
}

void
destroy_HertzMCdata (HertzMCdata *hzmc)
{
	gint i;


	//free everything
	g_free(hzmc->hertzmc);
	g_free(hzmc->mcavg);
	g_free(hzmc->mcstd);

	for (i = 0; i< NPARAM; i++){
		g_free(hzmc->mcdd[i]);
		g_free(hzmc->mcxhist[i]);
		g_free(hzmc->mcyhist[i]);
	}

	g_free(hzmc->mcdd);
	g_free(hzmc->mcxhist);
	g_free(hzmc->mcyhist);

	hzmc->mcnstat = 0;
	hzmc->skipmc = 0;
}

static gboolean 
nparam_to_string(gint i, gchar *str, gint nstr)
{

	switch (i){
		case 0:
			g_snprintf(str,nstr,"E_r/GPa");
			break;
		case 1:
			g_snprintf(str,nstr,"E_IT/GPa");
			break;
		default:
			printf("Unhandled number of results.\n");
			return FALSE;
	}
	return TRUE;
}


void hertz_mc_write_file(gchar *filename, Args *args) 
{
	FILE *file;
	Hertzdata *hz;
	gint i,j ;
	gchar str[200];
	gboolean is;

	if (args->fddata.hall->res == 0 || args->fddata.Fall->res==0) {
	    show_error("No data.");
	    return;
	}

	file = fopen(filename, "w");
	if (!file) {
	    show_error("Could not open file.");
	    return;
	}

	fprintf(file, "# %s \n", args->fddata.filename);
	fprintf(file, "#Monte Carlo simulation of Hertz analysis\n");
	fprintf(file, "#Input uncertainties \n");
	fprintf(file, "#u(h) = %g nm \n", args->hertzunc.uh);
	fprintf(file, "#u(F) = %g mN \n", args->hertzunc.uF);
	fprintf(file, "#Number of iterations  = %d  \n", args->hertzmc.Nmc);
	fprintf(file, "\n");

	fprintf(file, "#Parameters of Hertz analysis \n");
	fprintf(file, "#tip radius: %g nm\n", args->hertzdata.radius*1e9);
	fprintf(file, "#nu: %g                 \n", args->instdata.nu);
	fprintf(file, "#nu_indenter: %g        \n", args->instdata.nui);
	fprintf(file, "#E_indenter: %g      GPa\n", args->instdata.Ei * 1e-9); //TODO units
	fprintf(file, "#fitted range:  [%g, %g ] nm \n", args->hertzdata.range[0], args->hertzdata.range[1]);
	fprintf(file, "#\n");


	fprintf(file, "#\n\n");


	fprintf(file, "#Results \n");

	for (i = 0; i < NPARAM; i++){
		is = nparam_to_string(i, str, sizeof(str));
		fprintf(file, "#%s = (%g ± %g ) \n", str, args->hertzmc.mcavg[i], args->hertzmc.mcstd[i]);
	}

	fprintf(file, "\n\n\n");


	// Histograms

	if (args->hertzmc.mcnstat >0){
		fprintf(file,"#Histogram\n");
		for (i=0; i < NPARAM; i++){
			fprintf(file,"#Histogram\n");
			is = nparam_to_string(i, str, sizeof(str));
			if (is){
				fprintf(file,"#%s    count/a.u.  \n", str);
				for (j = 0;j < args->hertzmc.mcnstat; j++)
					fprintf(file, "%g %g \n", args->hertzmc.mcxhist[i][j], args->hertzmc.mcyhist[i][j]);
				fprintf(file, "\n \n");
			}
		}
	}


	fprintf(file, "\n\n\n");

	//Full data
	fprintf(file, "#Full data \n");
	fprintf(file, "#");
	for ( j = 0; j< NPARAM; j++){
		is = nparam_to_string(j, str, sizeof(str));
		fprintf(file, "%s ", str);
	}
	fprintf(file,"\n");

	hz= args->hertzmc.hertzmc;
	for (i = 0; i< args->hertzmc.Nmc; i++){
		fprintf(file, MODUL, hz->Er);
		fprintf(file,"  ");
		fprintf(file, MODUL, hz->Eit);
		fprintf(file,"  \n");
		hz++;
	}

	fclose(file); 
}



static void hertz_mc_hist_Er( Data *data)
{
	HertzMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=0;

	ctrls = data->controls->hertzmccontrols;

	printf("show Er histogram \n");
	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of E_r");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF E_r", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "E_r / GPa");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);



	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(E_r)",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->hertzmc.mcxhist[j], data->args->hertzmc.mcyhist[j], data->args->hertzmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);
}


static void hertz_mc_hist_Eit( Data *data)
{
	HertzMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=1;

	ctrls = data->controls->hertzmccontrols;

	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of E_IT");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF E_IT", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "E_IT / GPa");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);



	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(E_IT)",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->hertzmc.mcxhist[j], data->args->hertzmc.mcyhist[j], data->args->hertzmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);

}


static void hertz_mc_save_data( Args *args)
{
	GtkWidget *dialog;
	gchar *filename;

	dialog = gtk_file_chooser_dialog_new("Save File",
			NULL,
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_SAVE, GTK_RESPONSE_OK,
			NULL);

	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), gwy_app_get_current_directory());

	if ( gtk_dialog_run (GTK_DIALOG ( dialog)) == GTK_RESPONSE_OK)
	{
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		if (gwy_app_file_confirm_overwrite(dialog))
			hertz_mc_write_file(filename, args);
		g_free(filename);
	}
	gtk_widget_destroy ( dialog);
}

void 
hertz_mc_close_window(Data *data)
{

	gint j;

	if (data->controls->hertzmccontrols->has_window){
		gtk_widget_destroy(data->controls->hertzmccontrols->window);

		destroy_HertzMCdata(&(data->args->hertzmc));
		data->controls->hertzmccontrols->has_window = FALSE;
		for (j=0; j< NPARAM; j++){
			if (GTK_IS_WIDGET(data->controls->hertzmccontrols->hist_window[j])){
				gtk_widget_destroy(data->controls->hertzmccontrols->hist_window[j]);
			}
		}
		g_free(data->controls->hertzmccontrols->hist_window);
		g_free(data->controls->hertzmccontrols->button_hist);
	}
}

static void hertz_uncertainty_montecarlo_run_calc( Data *data)
{
	gint istart, iend, ndata;
	gint i, Nmc;
	Hertzdata *hz;
	HertzMCdata *mc;
	GwyDataLine *h, *F;
	GwyDataLine *hnew, *Fnew;
	GwyDataLine *hnoise, *Fnoise;
	GRand *rng;
	gint nn;
	gdouble **dd;
	gdouble *xhist, *yhist;
	gboolean hist;

	//not necessary to check range, couldn't get here in the first place

	//find appropriate data, which should be fitted, must use real fitted range
	range_to_indices(data->args->hertzdata.range[0], data->args->hertzdata.range[1], data->args->fddata.hload, FALSE, &istart, &iend, &ndata);
	// if there are not enough data in the selected range, not fitting can be performed
	ndata = MIN(ndata, data->args->fddata.hload->res - istart);

	if (ndata < 2){
		return;
	}

	//get data to be fitted
	h = gwy_data_line_part_extract(data->args->fddata.hload, istart, ndata);
	F = gwy_data_line_part_extract(data->args->fddata.Fload, istart, ndata);



	Nmc = data->args->hertzmc.Nmc;
	hz= data->args->hertzmc.hertzmc;
	//prepare data
	for (i=0; i< Nmc ;i++){
		init_Hertzdata(hz);
		hz++;
	}

	hz= data->args->hertzmc.hertzmc;
	//copy input parameters
	for (i=0; i< Nmc ;i++){
		hz->radius = data->args->hertzdata.radius;
		/* hz->nu = data->args->hertzdata.nu; */
		/* hz->nui = data->args->hertzdata.nui; */
		/* hz->Ei = data->args->hertzdata.Ei; */
		hz++;
	}

	/*
	for (i=0; i< Nmc ;i++){
		printf ("nu %g \n", data->args->hertzmc[i].nu);
	}
	*/

	// initialize random generator and datafield
	rng= g_rand_new();

	//prepare datafield for noise
	hnoise=gwy_data_line_new_alike(h,FALSE);
	Fnoise=gwy_data_line_new_alike(h,FALSE);

	//prepare datafield for varied fields
	hnew=gwy_data_line_new_alike(h,FALSE);
	Fnew=gwy_data_line_new_alike(F,FALSE);

	hz= data->args->hertzmc.hertzmc;
	for (i=0; i< Nmc; i++){

		//create noise for h-F data
		generate_uncorrelated_noise(hnoise, data->args->hertzunc.uh, rng);
		generate_uncorrelated_noise(Fnoise, data->args->hertzunc.uF, rng);

		//add noise to original h-F data
		sum_data_lines(hnew,h, hnoise);
		sum_data_lines(Fnew,F, Fnoise);

		//run fit
		hertz_fit(hnew, Fnew, hz, &(data->args->instdata));
/*		printf("-------------------\n");
		gint j;
		for (j=0; j< hnew->res;j++)
			printf("%g %g \n", hnew->data[j], Fnew->data[j]);
			*/
		hz++;
	}

	printf("run Monte Carlo \n");	
	g_object_unref(h);
	g_object_unref(F);
	g_object_unref(hnew);
	g_object_unref(Fnew);
	g_object_unref(hnoise);
	g_object_unref(Fnoise);
	g_rand_free(rng);

	for (i=0; i< Nmc; i++){
//		printf(" %d Er %g Eit %g \n", i, data->args->hertzmc[i].Er, data->args->hertzmc[i].Eit);
	}

	//calculate averages and stdevs
	// Er, Eit
	
	mc = &(data->args->hertzmc);
	for (i = 0; i< NPARAM; i++){
		mc->mcavg[i] =0;
		mc->mcstd[i] =0;
	}

	hz= data->args->hertzmc.hertzmc;
	nn = 0;
	for (i = 0; i< Nmc; i++){
		//sanity check
		{
//			printf("slope %g  Er %g \n", hz->reg.slope, hz->Er);
			mc->mcavg[0]+=hz->Er;
			mc->mcavg[1]+=hz->Eit;
			mc->mcstd[0]+=hz->Er*hz->Er;
			mc->mcstd[1]+=hz->Eit*hz->Eit;
			nn++;
		}
		hz++;
	}

	//use only reasonable data for average and standard deviation
	for (i = 0; i< NPARAM; i++){
		mc->mcavg[i] /= nn;
		mc->mcstd[i] /= nn;
		mc->mcstd[i]-= mc->mcavg[i]*mc->mcavg[i];
		mc->mcstd[i] = (mc->mcstd[i] < 0) ? 0 : mc->mcstd[i];
		mc->mcstd[i] = sqrt(mc->mcstd[i]);
	}

	// how many datasets were skipped
	data->args->hertzunc.skipmc = Nmc-nn;

	/*
	printf("mean  stdev \n");
	for (i = 0; i< NPARAM; i++){
		printf(" %g   %g \n", avg[i], std[i]);
	}
	*/
	 

	// Histograms
	//create data structure

	dd = data->args->hertzmc.mcdd;
	hz = data->args->hertzmc.hertzmc;
	for (i = 0; i< Nmc; i++){
		dd[0][i] = hz->Er;
		dd[1][i] = hz->Eit;
		hz++;
	}

	//prepare histogram structure

	for (i=0; i < NPARAM; i++){
		xhist = mc->mcxhist[i];
		yhist = mc->mcyhist[i];
		hist = create_histogram(dd[i], Nmc, xhist, yhist, mc->mcnstat );
		if (!hist){
			printf("Warning: No histogram could be created.\n");
			mc->mcnstat = -1;
			break;
		}
	}

}

void hertz_uncertainty_montecarlo( Data *data)
{

	gint i, j;
	GtkWidget *label;
	gchar str[200];
	GdkColor color;

	HertzMCControls *ctrls;
	HertzMCdata *hzmc;

	gdk_color_parse ("red", &color);

	//if there was a previous window, close it and any histograms, free data
	hertz_mc_close_window(data);

	hzmc = &(data->args->hertzmc);

	// initialize for number of iterations which was chosen in the Uncertainties dialog
	init_HertzMCdata(hzmc, data->args->hertzunc.Nmc); 

	// run calculation
	hertz_uncertainty_montecarlo_run_calc(data);

	//create window and display results
	ctrls = data ->controls->hertzmccontrols;

	ctrls->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->window), "Monte Carlo calculation of uncertainties in Hertz analysis");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->window), 400, 300); 
	g_signal_connect_swapped (ctrls->window, "delete_event", G_CALLBACK (hertz_mc_close_window), data);
	gtk_window_set_transient_for (GTK_WINDOW(ctrls->window), GTK_WINDOW(data->controls->maincontrols->window_main));

	ctrls->frame_input = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_input),
			GTK_WIDGET (gwy_label_new_header("Input for Monte Carlo simulation")));

	ctrls->table_input = gtk_table_new(3, 4, FALSE);

	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_input), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_input), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_input), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (ctrls->frame_input), ctrls->table_input);

	i = 0;

	/*          u(h)             */

	gtk_table_attach(GTK_TABLE(ctrls->table_input), gwy_label_new_header("u(h)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);

	label = gtk_label_new(NULL);
	g_snprintf(str,sizeof(str), "%g", data->args->hertzunc.uh);
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_input), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	gtk_table_attach(GTK_TABLE(ctrls->table_input), label_new_with_markup_left("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	gtk_table_attach(GTK_TABLE(ctrls->table_input), label_new_with_markup_left("normal distribution"), 3, 4, i, i+1, GTK_FILL, 0, 0, 0);

	i++;

	/*          u(F)             */

	gtk_table_attach(GTK_TABLE(ctrls->table_input), gwy_label_new_header("u(F)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);

	label = gtk_label_new(NULL);
	g_snprintf(str,sizeof(str), "%g", data->args->hertzunc.uF);
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_input), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	gtk_table_attach(GTK_TABLE(ctrls->table_input), label_new_with_markup_left("mN"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	gtk_table_attach(GTK_TABLE(ctrls->table_input), label_new_with_markup_left("normal distribution"), 3, 4, i, i+1, GTK_FILL, 0, 0, 0);

	i++;

	/*          Number of iteration */

	gtk_table_attach(GTK_TABLE(ctrls->table_input), gwy_label_new_header("Number of iterations "), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);

	label = gtk_label_new(NULL);
	g_snprintf(str,sizeof(str), "%d", hzmc->Nmc);
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_input), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	// Results
	ctrls->frame_results = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_results),
			GTK_WIDGET (gwy_label_new_header("Results of Monte Carlo simulation")));

	ctrls->table_results = gtk_table_new(NPARAM+1, 4, FALSE);

	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_results), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_results), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_results), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (ctrls->frame_results), ctrls->table_results);

	i = 0;

	gtk_table_attach(GTK_TABLE(ctrls->table_results), gwy_label_new_header("mean"), 1, 2, i, i+1, GTK_FILL, 0, 0, 0);
	gtk_table_attach(GTK_TABLE(ctrls->table_results), gwy_label_new_header("stdev"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	i++;

	ctrls->button_hist = (GtkWidget **)g_malloc(NPARAM*sizeof(GtkWidget *));
	ctrls->hist_window = (GtkWidget **)g_malloc(NPARAM*sizeof(GtkWidget *));
	for (j = 0; j< NPARAM; j++)
		ctrls->hist_window[j] = NULL;


	/*                       E_r           */

	gtk_table_attach(GTK_TABLE(ctrls->table_results),
			gwy_label_new_header("E<sub>r</sub>/GPa"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), MODUL, hzmc->mcavg[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), MODUL, hzmc->mcstd[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	ctrls->button_hist[i-1] = gtk_button_new();
	gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_SPECTRUM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (hertz_mc_hist_Er), data);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
	i++;

	/*                       E_IT          */

	gtk_table_attach(GTK_TABLE(ctrls->table_results),
			gwy_label_new_header("E<sub>it</sub>/GPa"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), MODUL, hzmc->mcavg[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	label = gtk_label_new(NULL);
	g_snprintf(str, sizeof(str), MODUL, hzmc->mcstd[i-1]); 
	gtk_label_set_text(GTK_LABEL(label),str);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	ctrls->button_hist[i-1] = gtk_button_new();
	gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_SPECTRUM, GTK_ICON_SIZE_BUTTON));
	g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (hertz_mc_hist_Eit), data);
	gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
	i++;

	printf(" Nmc %d \n", hzmc->Nmc);
	if (100.*data->args->hertzunc.skipmc/hzmc->Nmc > MAXBAD){
		label = gtk_label_new(NULL);
		g_snprintf(str, sizeof(str), "Warning: Approx %.1f %% of the results had to be removed!", 100.*data->args->hertzunc.skipmc/hzmc->Nmc);
		gtk_label_set_text(GTK_LABEL(label),str);
		gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &color);
		gtk_table_attach(GTK_TABLE(ctrls->table_results), label, 0,1, i, i+1, GTK_FILL, GTK_FILL,0,0);
		i++;
	}
	// save data
	ctrls->button_save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
	g_signal_connect_swapped (ctrls->button_save, "clicked", G_CALLBACK (hertz_mc_save_data), data->args);

	ctrls->vbox = gtk_vbox_new(FALSE, CTRLS_SPACING);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_input, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_results, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->button_save, FALSE, FALSE, 0);

	gtk_container_add(GTK_CONTAINER(ctrls->window), ctrls->vbox);
	gtk_widget_show_all(ctrls->window);
	ctrls->has_window = TRUE;

}
